@@ -96,40 +96,6 @@ class Linque(object):
         return any(condition(d) for d in self)
     
     
-    def argmax(self, key=lambda d: d):
-        """
-        Returns item having maximum value in current sequence by using default
-        comparer or specified item's key. This call fully evaluates current
-        sequence.
-        
-        Args:
-            key: callable
-                Item's key selector.
-        
-        Returns:
-            any
-        """
-        
-        return max(self, key=key)
-    
-    
-    def argmin(self, key=lambda d: d):
-        """
-        Returns item having minimum value in current sequence by using default
-        comparer or specified item's key. This call fully evaluates current
-        sequence.
-        
-        Args:
-            key: callable
-                Item's key selector.
-        
-        Returns:
-            any
-        """
-        
-        return min(self, key=key)
-    
-    
     def chunk(self, size):
         """
         Splits current sequence into chunks of specified size. This call does
@@ -200,38 +166,26 @@ class Linque(object):
         return iters.count(self, condition)
     
     
-    def dict(self, key, value=lambda d: d):
-        """
-        Evaluates items into dictionary. This call fully evaluates current
-        sequence.
-        
-        Args:
-            key: callable
-                Item's key selector.
-            
-            value: callable
-                Item's value selector.
-        
-        Returns:
-            dict
-        """
-        
-        result = {}
-        
-        for item in self:
-            k = key(item)
-            if k in result:
-                raise KeyError("Key is not unique.")
-            result[k] = value(item)
-        
-        return result
-    
-    
-    def distinct(self, key=lambda d: d):
+    def distinct(self):
         """
         Produces new sequence by selecting distinct items from current sequence
-        using default comparer or specified item's key. First occurrence of each
-        item is used. This call does not evaluate current sequence.
+        using default comparer. First occurrence of each item is used. This call
+        does not evaluate current sequence.
+        
+        Returns:
+            Linque
+        """
+        
+        result = iters.distinct(self)
+        
+        return Linque(result, self._evaluate)
+    
+    
+    def distinct_by(self, key):
+        """
+        Produces new sequence by selecting distinct items from current sequence
+        using specified item's key. First occurrence of each item is used. This
+        call does not evaluate current sequence.
         
         Args:
             key: callable
@@ -241,7 +195,7 @@ class Linque(object):
             Linque
         """
         
-        result = iters.distinct(self, key)
+        result = iters.distinct_by(self, key)
         
         return Linque(result, self._evaluate)
     
@@ -299,11 +253,28 @@ class Linque(object):
         return self
     
     
-    def exclude(self, items, key=lambda d: d):
+    def exclude(self, items):
         """
         Produces new sequence by excluding specified items from current sequence
-        using default comparer or selected item's key. This call does not
-        evaluate current sequence.
+        using default comparer. This call does not evaluate current sequence.
+        
+        Args:
+            items: (any,)
+                Items to exclude.
+        
+        Returns:
+            Linque
+        """
+        
+        result = iters.exclude(self, items)
+        
+        return Linque(result, self._evaluate)
+    
+    
+    def exclude_by(self, items, key):
+        """
+        Produces new sequence by excluding specified items from current sequence
+        using selected item's key. This call does not evaluate current sequence.
         
         Args:
             items: (any,)
@@ -316,12 +287,29 @@ class Linque(object):
             Linque
         """
         
-        result = iters.exclude(self, items, key)
+        result = iters.exclude_by(self, items, key)
         
         return Linque(result, self._evaluate)
     
     
-    def first(self, condition=None, default=None):
+    def first(self, condition=None):
+        """
+        Returns the first item in current sequence that satisfies specified
+        condition or raises error if no item found. This call partially
+        evaluates current sequence.
+        
+        Args:
+            condition: callable or None
+                Condition to test.
+        
+        Returns:
+            any
+        """
+        
+        return iters.first(self, condition)
+    
+    
+    def first_or_default(self, condition=None, default=None):
         """
         Returns the first item in current sequence that satisfies specified
         condition or specified default value if no item found. This call
@@ -338,7 +326,7 @@ class Linque(object):
             any
         """
         
-        return iters.first(self, condition, default)
+        return iters.first_or_default(self, condition, default)
     
     
     def flatten(self, selector=None):
@@ -362,7 +350,23 @@ class Linque(object):
         return Linque(result, self._evaluate)
     
     
-    def group(self, key=lambda d: d):
+    def group(self):
+        """
+        Produces new sequence by grouping items of current sequence according to
+        default comparer and creates result values as (key, group) pairs.
+        This call fully evaluates current sequence.
+        
+        Returns:
+            Linque
+        """
+        
+        groups = iters.group(self)
+        result = [(k, Linque(g, self._evaluate)) for k, g in groups]
+        
+        return Linque(result, self._evaluate)
+    
+    
+    def group_by(self, key):
         """
         Produces new sequence by grouping items of current sequence according to
         specified key selector and creates result values as (key, group) pairs.
@@ -376,21 +380,40 @@ class Linque(object):
             Linque
         """
         
-        groups = iters.group(self, key)
+        groups = iters.group_by(self, key)
         result = [(k, Linque(g, self._evaluate)) for k, g in groups]
         
         return Linque(result, self._evaluate)
     
     
-    def intersect(self, items, key=lambda d: d):
+    def intersect(self, items):
         """
         Produces new sequence of shared unique items from current sequence and
-        given items by using default comparer or selected item's key. This call
-        does not evaluate current sequence.
+        given items by using default comparer. This call does not evaluate
+        current sequence.
         
         Args:
             items: (any,)
-                Items to union.
+                Items to intersect.
+        
+        Returns:
+            Linque
+        """
+        
+        result = iters.intersect(self, items)
+        
+        return Linque(result, self._evaluate)
+    
+    
+    def intersect_by(self, items, key):
+        """
+        Produces new sequence of shared unique items from current sequence and
+        given items by using selected item's key. This call does not evaluate
+        current sequence.
+        
+        Args:
+            items: (any,)
+                Items to intersect.
             
             key: callable
                 Item's key selector.
@@ -399,12 +422,29 @@ class Linque(object):
             Linque
         """
         
-        result = iters.intersect(self, items, key)
+        result = iters.intersect_by(self, items, key)
         
         return Linque(result, self._evaluate)
     
     
-    def last(self, condition=None, default=None):
+    def last(self, condition=None):
+        """
+        Returns the last item in current sequence that satisfies specified
+        condition or raises error if no item found. This call fully evaluates
+        current sequence.
+        
+        Args:
+            condition: callable or None
+                Condition to test.
+        
+        Returns:
+            any
+        """
+        
+        return iters.last(self, condition)
+    
+    
+    def last_or_default(self, condition=None, default=None):
         """
         Returns the last item in current sequence that satisfies specified
         condition or specified default value if no item found. This call
@@ -421,18 +461,7 @@ class Linque(object):
             any
         """
         
-        return iters.last(self, condition, default)
-    
-    
-    def list(self):
-        """
-        Evaluate items into list. This call fully evaluates current sequence.
-        
-        Returns:
-            list
-        """
-        
-        return list(self)
+        return iters.last_or_default(self, condition, default)
     
     
     def max(self, selector=None):
@@ -454,6 +483,23 @@ class Linque(object):
         return max(selector(d) for d in self)
     
     
+    def max_by(self, key):
+        """
+        Returns item having maximum value in current sequence by using default
+        comparer or specified item's key. This call fully evaluates current
+        sequence.
+        
+        Args:
+            key: callable
+                Item's key selector.
+        
+        Returns:
+            any
+        """
+        
+        return max(self, key=key)
+    
+    
     def mean(self, selector=None):
         """
         Returns average value of current sequence by specified items data
@@ -511,6 +557,23 @@ class Linque(object):
         return min(selector(d) for d in self)
     
     
+    def min_by(self, key):
+        """
+        Returns item having minimum value in current sequence by using default
+        comparer or specified item's key. This call fully evaluates current
+        sequence.
+        
+        Args:
+            key: callable
+                Item's key selector.
+        
+        Returns:
+            any
+        """
+        
+        return min(self, key=key)
+    
+    
     def reverse(self):
         """
         Produces new sequence by inverting order of items in current sequence.
@@ -564,15 +627,21 @@ class Linque(object):
         return Linque(result, self._evaluate)
     
     
-    def set(self):
+    def single(self, condition=None):
         """
-        Evaluate items into set. This call fully evaluates current sequence.
+        Returns the single item in current sequence that satisfies specified
+        condition or raises error if none or more items found. This call fully
+        evaluates current sequence.
+        
+        Args:
+            condition: callable
+                Condition to test.
         
         Returns:
-            set
+            any
         """
         
-        return set(self)
+        return iters.single(self, condition)
     
     
     def skip(self, count):
@@ -612,11 +681,40 @@ class Linque(object):
         return Linque(result, self._evaluate)
     
     
-    def sort(self, key=lambda d: d):
+    def sort(self):
         """
         Produces new sequence by sorting elements of current sequence in
-        ascending order by using default comparer or selected item's key. This
-        call fully evaluates current sequence.
+        ascending order by using default comparer. This call fully evaluates
+        current sequence.
+        
+        Returns:
+            Linque
+        """
+        
+        result = sorted(self)
+        
+        return Linque(result, self._evaluate)
+    
+    
+    def sort_desc(self):
+        """
+        Sorts elements of current sequence in descending order by using default
+        comparer. This call fully evaluates current sequence.
+        
+        Returns:
+            Linque
+        """
+        
+        result = sorted(self, reverse=True)
+        
+        return Linque(result, self._evaluate)
+    
+    
+    def sort_by(self, key):
+        """
+        Produces new sequence by sorting elements of current sequence in
+        ascending order by using selected item's key. This call fully evaluates
+        current sequence.
         
         Args:
             key: callable
@@ -631,11 +729,10 @@ class Linque(object):
         return Linque(result, self._evaluate)
     
     
-    def sort_desc(self, key=lambda d: d):
+    def sort_by_desc(self, key):
         """
-        Sorts elements of current sequence in descending order by using default
-        comparer or selected item's key. This call fully evaluates current
-        sequence.
+        Sorts elements of current sequence in descending order by using selected
+        item's key. This call fully evaluates current sequence.
         
         Args:
             key: callable
@@ -707,7 +804,56 @@ class Linque(object):
         return Linque(result, self._evaluate)
     
     
-    def tuple(self):
+    def to_dict(self, key, value=lambda d: d):
+        """
+        Evaluates items into dictionary. This call fully evaluates current
+        sequence.
+        
+        Args:
+            key: callable
+                Item's key selector.
+            
+            value: callable
+                Item's value selector.
+        
+        Returns:
+            dict
+        """
+        
+        result = {}
+        
+        for item in self:
+            k = key(item)
+            if k in result:
+                raise KeyError("Key is not unique.")
+            result[k] = value(item)
+        
+        return result
+    
+    
+    def to_list(self):
+        """
+        Evaluate items into list. This call fully evaluates current sequence.
+        
+        Returns:
+            list
+        """
+        
+        return list(self)
+    
+    
+    def to_set(self):
+        """
+        Evaluate items into set. This call fully evaluates current sequence.
+        
+        Returns:
+            set
+        """
+        
+        return set(self)
+    
+    
+    def to_tuple(self):
         """
         Evaluate items into tuple. This call fully evaluates current sequence.
         
@@ -718,11 +864,30 @@ class Linque(object):
         return tuple(self)
     
     
-    def union(self, items, key=lambda d: d):
+    def union(self, items):
         """
         Produces new sequence of unique items from current sequence and given
-        items by using default comparer or selected item's key. This call does
-        not evaluate current sequence.
+        items by using default comparer. This call does not evaluate current
+        sequence.
+        
+        Args:
+            items: (any,)
+                Items to union.
+        
+        Returns:
+            Linque
+        """
+        
+        result = iters.union(self, items)
+        
+        return Linque(result, self._evaluate)
+    
+    
+    def union_by(self, items, key):
+        """
+        Produces new sequence of unique items from current sequence and given
+        items by using selected item's key. This call does not evaluate current
+        sequence.
         
         Args:
             items: (any,)
@@ -735,7 +900,7 @@ class Linque(object):
             Linque
         """
         
-        result = iters.union(self, items, key)
+        result = iters.union_by(self, items, key)
         
         return Linque(result, self._evaluate)
     
